
# q-expansion of newform 2128.2.a.b, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2128
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [-1, -1], [2, -3], [-1, 0], [-3, -1], [2, 2], [-4, 4], [1, 0], [-4, 6], [-4, -3], [-6, 4], [0, 3], [-6, 7], [6, 1], [-1, 3], [-13, 1], [4, 3], [9, -7], [2, -8], [5, -13], [-4, -2], [2, 7], [4, -8], [-3, 1], [-1, 3], [2, 8], [-12, 2], [4, -2], [3, -7], [4, -8], [13, -1], [-10, 10], [-12, 1], [-4, -4], [4, 0], [-4, 16], [-11, 5], [1, -5], [10, -8], [-8, -6], [-16, -2], [12, -10], [0, -14], [-4, -6], [2, -8], [-4, -13], [4, 0], [-10, 14], [-16, 8], [2, 13], [-15, 19], [-14, 18], [14, -3], [-10, -2], [10, -21], [6, 2], [22, 4], [-4, -3], [-12, 14], [16, 0], [-12, -4], [10, 10], [4, 13], [-1, -1], [8, -16], [-19, -1], [22, -12], [-14, -2], [-12, -12], [-6, 0], [24, -20], [-24, 6], [-9, 7], [12, -19], [-30, 14], [24, -2], [-10, 20], [13, 5], [24, -8], [-13, -9], [2, -12], [18, -20], [19, -15], [14, 5], [2, -4], [-10, 11], [-14, 26], [5, -17], [-7, 1], [6, -2], [-12, -6], [24, -15], [-22, -9], [-28, 16], [14, 7], [4, 5], [32, 6], [-6, -4], [-24, 0], [12, 6], [8, -2], [-22, -2], [5, -3], [-20, 6], [7, 13], [-22, -8], [-8, -2], [-32, 6], [-3, -9], [-18, 0], [26, -10], [-12, 32], [-40, 3], [28, -26], [0, -20], [-32, 10], [-4, -4], [-44, 7], [22, -6], [20, -16], [22, -16], [2, 14], [6, -16], [34, 0], [-8, -18], [20, -14], [2, -6], [-16, -16], [15, -17], [6, -3], [-6, -11], [-34, -7], [10, 1], [-28, 12], [-22, -16], [-16, -10], [28, -6], [-24, 25], [16, -42], [4, -19], [-43, 1], [-2, 12], [2, -20], [-16, -10], [20, -12], [-30, 16], [-6, 31], [-14, 16], [20, -18], [-14, 13], [-1, -15], [6, -2], [-3, -9], [-10, 36], [10, -26], [26, -23], [-28, 2], [10, -6], [8, 24], [-34, 26], [-21, 1], [2, 4], [-36, 28], [43, -17], [32, -2], [12, -8], [9, 27], [26, -5], [-26, 10], [-26, -4], [28, 14], [32, -10], [28, -42], [16, -7], [2, 36], [-42, -4], [-10, 15], [-24, 2], [26, -4], [14, -31], [-4, -6], [-17, 19], [11, -11], [-11, 13], [-24, -14], [-15, 5], [-52, 5], [-32, 7], [8, 30], [1, 33], [-10, -4], [-30, 25], [26, -36], [-42, 10], [-55, 7], [-18, -17], [51, 5], [-18, 22], [-24, 16], [36, -7], [22, 12], [0, -2], [-2, -2], [-25, 27], [56, -12], [3, -29], [-37, -13], [10, -4], [-2, 4], [13, -15], [-4, -1], [5, 25], [50, -10], [-48, 31], [10, 10], [2, 35], [-36, 50], [-22, -27], [-46, 39], [0, 32], [39, -31], [2, -42], [8, -40], [16, -20], [-3, -11], [38, -40], [-6, -5], [-32, 56], [-52, 20], [-38, 19], [-50, 18], [31, 23], [-5, -35], [-14, 34], [-16, 38], [0, 33], [-20, 20], [-42, 20], [22, -15], [32, -26], [26, -30], [52, -4], [-16, -24], [-13, 29], [-26, 36], [57, -7], [4, 7], [8, -12], [-26, 22], [-32, 4], [-44, 56], [50, -20], [54, -10], [-42, -8], [54, -24], [-10, 39], [-16, -8], [-43, -3], [-8, 12], [10, -44], [2, 29], [4, 10], [25, 5], [-44, 18], [22, -30], [5, 13], [62, -12], [64, -6], [48, 0], [14, -30], [-30, 8], [8, -8], [-16, 54], [18, -42], [-2, 45], [12, 39], [-32, -16], [0, 7], [-36, 36], [-4, 42], [26, -36], [-16, -4], [37, -35], [54, -14], [16, -44], [-42, 2], [1, 7], [12, -33], [47, 23], [-2, -36], [-12, -20], [34, -48], [-28, 46], [36, -9], [42, 8], [-18, 43], [0, -14], [-22, 31], [-67, 17], [-20, 32], [-6, -14], [-22, 4], [-42, -12], [-12, 46], [-42, 13], [8, -40], [-10, 2], [-36, 36], [20, -52], [20, -28], [-28, 39], [2, -4], [16, -60], [76, -8], [-12, 62], [-58, 37], [-42, 40], [14, 17], [34, 32], [7, 5], [-29, 31], [-4, -7], [42, 16], [29, -45], [-42, 12], [30, 25], [-18, 38], [28, 5], [-67, 35], [-30, 30], [-30, -36], [40, -1], [49, -59], [-48, 24], [-10, -13], [0, -16], [3, 37], [-11, -39], [-10, 4], [5, 1], [-71, 11], [-43, 7], [-26, 32], [84, -8], [28, -43], [-18, -30], [39, -45], [-10, -28], [2, 0], [18, 3], [12, -17], [-46, 8], [-28, -24], [58, 10], [12, -19], [27, -41], [-29, 15], [38, 12], [3, -27], [-23, 23], [0, -6], [66, -12], [16, 4], [-2, 34], [6, -12], [-70, 24], [14, -27], [22, -31], [40, -36], [30, -11], [-8, -3], [-24, -30], [42, 4], [16, -8], [40, -36], [-13, -1], [82, 0], [-62, 10], [2, -44], [42, -43], [10, -16], [-23, -17], [-64, 34], [-56, 23], [-31, 9], [60, -38], [22, 32], [6, 39], [-36, 24], [-60, 42], [6, -4], [8, 48], [-58, 63], [24, -32], [-22, -25], [-8, -8], [-12, 30], [7, -13], [0, -16], [60, -68], [-34, -20], [-47, 47], [7, 13], [57, 25], [0, 24], [-36, 37], [52, -40], [26, 28], [12, -48], [-20, 8], [27, 15], [18, 14], [-66, 44], [-23, 51], [-84, 27], [37, -43], [-24, 28], [12, -18], [-82, 28], [0, -22], [19, 37], [-16, -24], [-14, 0], [88, -20], [-7, -59], [-18, 26], [-88, 30], [-12, 69], [66, -2], [12, 16]]
